from django.conf import settings
from netbox.plugins import PluginMenuItem

plugin_settings = settings.PLUGINS_CONFIG.get("netbox_old_search", {})

if plugin_settings.get("show_menu"):
    menu_items = (
        PluginMenuItem(
            link="plugins:netbox_old_search:customsearch",
            link_text="Old Search",
        ),
    )



from django.conf import settings
from django.shortcuts import redirect
from django.urls import reverse

plugin_settings = settings.PLUGINS_CONFIG.get("netbox_old_search", {})


class SearchRedirectMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)

        if plugin_settings.get("replace_search") and request.path.startswith("/search/"):
            redirect_url = reverse("plugins:netbox_old_search:customsearch")

            return redirect(f"{redirect_url}?{request.GET.urlencode()}")

        return response



from django.shortcuts import redirect, render
from django.urls import reverse
from django.utils.translation import gettext as _
from django.views.generic import View
from .forms import SearchForm
from .search import SEARCH_TYPES

SEARCH_MAX_RESULTS = 100


class SearchView(View):
    def get(self, request):
        form = SearchForm(request.GET)
        results = []

        if form.is_valid():
            # If an object type has been specified, redirect to the dedicated view for it
            if form.cleaned_data["obj_type"]:
                object_type = form.cleaned_data["obj_type"]
                url = reverse(SEARCH_TYPES[object_type]["url"])
                return redirect(f"{url}?q={form.cleaned_data['q']}")

            for obj_type in SEARCH_TYPES.keys():
                queryset = SEARCH_TYPES[obj_type]["queryset"].restrict(request.user, "view")
                filterset = SEARCH_TYPES[obj_type]["filterset"]
                table = SEARCH_TYPES[obj_type]["table"]
                url = SEARCH_TYPES[obj_type]["url"]

                # Construct the results table for this object type
                filtered_queryset = filterset({"q": form.cleaned_data["q"]}, queryset=queryset).qs
                table = table(filtered_queryset, orderable=False)
                table.paginate(per_page=SEARCH_MAX_RESULTS)

                if table.page:
                    results.append({"name": queryset.model._meta.verbose_name_plural, "table": table, "url": f"{reverse(url)}?q={form.cleaned_data.get('q')}"})

        return render(
            request,
            "netbox_old_search/search.html",
            {
                "form": form,
                "results": results,
            },
        )



from netbox.plugins import PluginConfig
from .middleware import SearchRedirectMiddleware


class NetboxOldSearch(PluginConfig):
    name = "netbox_old_search"
    verbose_name = "Old Search"
    description = "Netbox Old Search"
    version = "1.1.0"
    base_url = "old-search"
    default_settings = {"show_menu": False, "replace_search": True}
    middleware = ["netbox_old_search.middleware.SearchRedirectMiddleware"]


config = NetboxOldSearch



from django import forms

from .search import SEARCH_TYPE_HIERARCHY


def build_search_choices():
    result = list()
    result.append(("", "All Objects"))
    for category, items in SEARCH_TYPE_HIERARCHY.items():
        subcategories = list()
        for slug, obj in items.items():
            name = obj["queryset"].model._meta.verbose_name_plural
            name = name[0].upper() + name[1:]
            subcategories.append((slug, name))
        result.append((category, tuple(subcategories)))

    return tuple(result)


OBJ_TYPE_CHOICES = build_search_choices()


def build_options():
    options = [{"label": OBJ_TYPE_CHOICES[0][1], "items": []}]

    for label, choices in OBJ_TYPE_CHOICES[1:]:
        items = []

        for value, choice_label in choices:
            items.append({"label": choice_label, "value": value})

        options.append({"label": label, "items": items})
    return options


class SearchForm(forms.Form):
    q = forms.CharField(label="Search")
    obj_type = forms.ChoiceField(choices=OBJ_TYPE_CHOICES, required=False, label="Type")
    options = build_options()



import circuits.filtersets
import circuits.tables
import dcim.filtersets
import dcim.tables
import extras.filtersets
import extras.tables
import ipam.filtersets
import ipam.tables
import tenancy.filtersets
import tenancy.tables
import virtualization.filtersets
import wireless.tables
import wireless.filtersets
import virtualization.tables
from circuits.models import Circuit, ProviderNetwork, Provider
from dcim.models import (
    Cable,
    Device,
    DeviceType,
    Interface,
    Location,
    Module,
    ModuleType,
    PowerFeed,
    Rack,
    RackReservation,
    Site,
    VirtualChassis,
)
from extras.models import JournalEntry
from ipam.models import Aggregate, ASN, IPAddress, Prefix, Service, VLAN, VRF
from netbox.settings import VERSION
from tenancy.models import Contact, Tenant, ContactAssignment
from wireless.models import WirelessLAN, WirelessLink
from virtualization.models import Cluster, VirtualMachine
from utilities.query import count_related


NB_VERSION = [int(n) for n in VERSION.split("-")[0].split(".")]

# Netbox 3.6.0 implemented cached counters for VC's.
VC_QUERY = None
if NB_VERSION >= [3, 6, 0]:
    VC_QUERY = VirtualChassis.objects.prefetch_related("master")
else:
    VC_QUERY = VirtualChassis.objects.prefetch_related("master").annotate(member_count=count_related(Device, "virtual_chassis"))

CIRCUIT_TYPES = {
    "provider": {
        "queryset": Provider.objects.prefetch_related("circuits", "accounts").annotate(count_circuits=count_related(Circuit, "provider")),
        "filterset": circuits.filtersets.ProviderFilterSet,
        "table": circuits.tables.ProviderTable,
        "url": "circuits:provider_list",
    },
    "circuit": {
        "queryset": Circuit.objects.prefetch_related("type", "provider", "tenant", "tenant__group", "terminations__site"),
        "filterset": circuits.filtersets.CircuitFilterSet,
        "table": circuits.tables.CircuitTable,
        "url": "circuits:circuit_list",
    },
    "providernetwork": {
        "queryset": ProviderNetwork.objects.prefetch_related("provider"),
        "filterset": circuits.filtersets.ProviderNetworkFilterSet,
        "table": circuits.tables.ProviderNetworkTable,
        "url": "circuits:providernetwork_list",
    },
}

DCIM_TYPES = {
    "site": {
        "queryset": Site.objects.prefetch_related("region", "tenant", "tenant__group"),
        "filterset": dcim.filtersets.SiteFilterSet,
        "table": dcim.tables.SiteTable,
        "url": "dcim:site_list",
    },
    "rack": {
        "queryset": Rack.objects.prefetch_related("site", "location", "tenant", "tenant__group", "role").annotate(
            device_count=count_related(Device, "rack")
        ),
        "filterset": dcim.filtersets.RackFilterSet,
        "table": dcim.tables.RackTable,
        "url": "dcim:rack_list",
    },
    "rackreservation": {
        "queryset": RackReservation.objects.prefetch_related("rack", "user"),
        "filterset": dcim.filtersets.RackReservationFilterSet,
        "table": dcim.tables.RackReservationTable,
        "url": "dcim:rackreservation_list",
    },
    "location": {
        "queryset": Location.objects.add_related_count(
            Location.objects.add_related_count(
                Location.objects.all(),
                Device,
                "location",
                "device_count",
                cumulative=True,
            ),
            Rack,
            "location",
            "rack_count",
            cumulative=True,
        ).prefetch_related("site"),
        "filterset": dcim.filtersets.LocationFilterSet,
        "table": dcim.tables.LocationTable,
        "url": "dcim:location_list",
    },
    "devicetype": {
        "queryset": DeviceType.objects.prefetch_related("manufacturer").annotate(instance_count=count_related(Device, "device_type")),
        "filterset": dcim.filtersets.DeviceTypeFilterSet,
        "table": dcim.tables.DeviceTypeTable,
        "url": "dcim:devicetype_list",
    },
    "device": {
        "queryset": Device.objects.prefetch_related(
            "device_type__manufacturer",
            "tenant",
            "tenant__group",
            "site",
            "rack",
            "primary_ip4",
            "primary_ip6",
        ),
        "filterset": dcim.filtersets.DeviceFilterSet,
        "table": dcim.tables.DeviceTable,
        "url": "dcim:device_list",
    },
    "moduletype": {
        "queryset": ModuleType.objects.prefetch_related("manufacturer").annotate(instance_count=count_related(Module, "module_type")),
        "filterset": dcim.filtersets.ModuleTypeFilterSet,
        "table": dcim.tables.ModuleTypeTable,
        "url": "dcim:moduletype_list",
    },
    "module": {
        "queryset": Module.objects.prefetch_related(
            "module_type__manufacturer",
            "device",
            "module_bay",
        ),
        "filterset": dcim.filtersets.ModuleFilterSet,
        "table": dcim.tables.ModuleTable,
        "url": "dcim:module_list",
    },
    "virtualchassis": {
        "queryset": VC_QUERY,
        "filterset": dcim.filtersets.VirtualChassisFilterSet,
        "table": dcim.tables.VirtualChassisTable,
        "url": "dcim:virtualchassis_list",
    },
    "cable": {
        "queryset": Cable.objects.all(),
        "filterset": dcim.filtersets.CableFilterSet,
        "table": dcim.tables.CableTable,
        "url": "dcim:cable_list",
    },
    "powerfeed": {
        "queryset": PowerFeed.objects.all(),
        "filterset": dcim.filtersets.PowerFeedFilterSet,
        "table": dcim.tables.PowerFeedTable,
        "url": "dcim:powerfeed_list",
    },
}

IPAM_TYPES = {
    "vrf": {
        "queryset": VRF.objects.prefetch_related("tenant", "tenant__group"),
        "filterset": ipam.filtersets.VRFFilterSet,
        "table": ipam.tables.VRFTable,
        "url": "ipam:vrf_list",
    },
    "aggregate": {
        "queryset": Aggregate.objects.prefetch_related("rir"),
        "filterset": ipam.filtersets.AggregateFilterSet,
        "table": ipam.tables.AggregateTable,
        "url": "ipam:aggregate_list",
    },
    "prefix": {
        "queryset": Prefix.objects.prefetch_related("scope", "vrf__tenant", "tenant", "tenant__group", "vlan", "role"),
        "filterset": ipam.filtersets.PrefixFilterSet,
        "table": ipam.tables.PrefixTable,
        "url": "ipam:prefix_list",
    },
    "ipaddress": {
        "queryset": IPAddress.objects.prefetch_related("vrf__tenant", "tenant", "tenant__group"),
        "filterset": ipam.filtersets.IPAddressFilterSet,
        "table": ipam.tables.IPAddressTable,
        "url": "ipam:ipaddress_list",
    },
    "vlan": {
        "queryset": VLAN.objects.prefetch_related("site", "group", "tenant", "tenant__group", "role"),
        "filterset": ipam.filtersets.VLANFilterSet,
        "table": ipam.tables.VLANTable,
        "url": "ipam:vlan_list",
    },
    "asn": {
        "queryset": ASN.objects.prefetch_related("rir", "tenant", "tenant__group"),
        "filterset": ipam.filtersets.ASNFilterSet,
        "table": ipam.tables.ASNTable,
        "url": "ipam:asn_list",
    },
    "service": {
        "queryset": Service.objects.prefetch_related("device", "virtual_machine"),
        "filterset": ipam.filtersets.ServiceFilterSet,
        "table": ipam.tables.ServiceTable,
        "url": "ipam:service_list",
    },
}

TENANCY_TYPES = {
    "tenant": {
        "queryset": Tenant.objects.prefetch_related("group"),
        "filterset": tenancy.filtersets.TenantFilterSet,
        "table": tenancy.tables.TenantTable,
        "url": "tenancy:tenant_list",
    },
    "contact": {
        "queryset": Contact.objects.prefetch_related("group", "assignments").annotate(
            assignment_count=count_related(ContactAssignment, "contact")
        ),
        "filterset": tenancy.filtersets.ContactFilterSet,
        "table": tenancy.tables.ContactTable,
        "url": "tenancy:contact_list",
    },
}

VIRTUALIZATION_TYPES = {
    "cluster": {
        "queryset": Cluster.objects.prefetch_related("type", "group").annotate(
            device_count=count_related(Device, "cluster"),
            vm_count=count_related(VirtualMachine, "cluster"),
        ),
        "filterset": virtualization.filtersets.ClusterFilterSet,
        "table": virtualization.tables.ClusterTable,
        "url": "virtualization:cluster_list",
    },
    "virtualmachine": {
        "queryset": VirtualMachine.objects.prefetch_related(
            "cluster",
            "tenant",
            "tenant__group",
            "platform",
            "primary_ip4",
            "primary_ip6",
        ),
        "filterset": virtualization.filtersets.VirtualMachineFilterSet,
        "table": virtualization.tables.VirtualMachineTable,
        "url": "virtualization:virtualmachine_list",
    },
}

WIRELESS_TYPES = {
    "wirelesslan": {
        "queryset": WirelessLAN.objects.prefetch_related("group", "vlan").annotate(
            interface_count=count_related(Interface, "wireless_lans")
        ),
        "filterset": wireless.filtersets.WirelessLANFilterSet,
        "table": wireless.tables.WirelessLANTable,
        "url": "wireless:wirelesslan_list",
    },
    "wirelesslink": {
        "queryset": WirelessLink.objects.prefetch_related("interface_a__device", "interface_b__device"),
        "filterset": wireless.filtersets.WirelessLinkFilterSet,
        "table": wireless.tables.WirelessLinkTable,
        "url": "wireless:wirelesslink_list",
    },
}

JOURNAL_TYPES = {
    "journalentry": {
        "queryset": JournalEntry.objects.prefetch_related("assigned_object", "created_by"),
        "filterset": extras.filtersets.JournalEntryFilterSet,
        "table": extras.tables.JournalEntryTable,
        "url": "extras:journalentry_list",
    },
}

SEARCH_TYPE_HIERARCHY = {
    "Circuits": CIRCUIT_TYPES,
    "DCIM": DCIM_TYPES,
    "IPAM": IPAM_TYPES,
    "Tenancy": TENANCY_TYPES,
    "Virtualization": VIRTUALIZATION_TYPES,
    "Wireless": WIRELESS_TYPES,
    "Journal": JOURNAL_TYPES,
}


def build_search_types():
    result = dict()

    for app_types in SEARCH_TYPE_HIERARCHY.values():
        for name, items in app_types.items():
            result[name] = items

    return result


SEARCH_TYPES = build_search_types()



#!/usr/bin/env python
from setuptools import setup, find_packages

with open("README.md") as readme_file:
    readme = readme_file.read()

requirements = []

setup(
    author="kkthxbye-code",
    author_email="festll234@gmail.com",
    python_requires=">=3.8",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    description="Brings back the old search functionality to NetBox",
    install_requires=requirements,
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords=["netbox", "netbox-plugin"],
    name="netbox-old-search",
    packages=find_packages(include=["netbox_old_search", "netbox_old_search.*"]),
    url="https://github.com/kkthxbye-code/netbox-old-search",
    version="1.1.0",
    zip_safe=False,
)
